Rem
Rem  odbccat10.sql
Rem
Rem  $Id: odbccat10.sql,v 1.4 2012/06/29 00:23:14 source Exp $
Rem
Rem  OpenLink ODBC Catalog views for Oracle 10 RDBMS
Rem
Rem  These catalog views provide the ODBC catalog for the
Rem  SQLForeignKeys and SQLPrimaryKeys functions.
Rem  If you plan to use these functions from ODBC, please install
Rem  these views. Refer to your OpenLink database agent documentation
Rem  for further details.
Rem
Rem  Copyright (C) 2004-2012 OpenLink Software.
Rem  All Rights Reserved.
Rem
Rem  The copyright above and this notice must be preserved in all
Rem  copies of this source code.  The copyright above does not
Rem  evidence any actual or intended publication of this source code.
Rem
Rem  This is unpublished proprietary trade secret source code of
Rem  OpenLink Software.  This source code may not be copied, disclosed,
Rem  distributed, demonstrated or licensed except as expressly 
Rem  authorized by OpenLink Software.
Rem
Rem
Rem NOTICE:
Rem
Rem This script must be run connected to the internal database
Rem in order to register properly. As this might require the DBA
Rem to enter a password this script needs to be run as follows:
Rem 
Rem 	log in as the Oracle DBA
Rem 	$ svrmgrl 
Rem	SQL> connect internal
Rem	enter password when requested by Oracle
Rem	SQL> @odbccat10.sql
Rem     ...........
Rem         <output from running Oracle script deleted>
Rem	...........
Rem	SQL> quit
Rem
Rem OR
Rem
Rem     $ sqlplus 
Rem     SQL> connect system as sysdba
Rem	enter password when requested by Oracle
Rem	SQL> @odbccat10.sql
Rem     ...........
Rem         <output from running Oracle script deleted>
Rem	...........
Rem	SQL> quit
Rem
Rem

create or replace view OPENLINK$SQL_FOREIGN_KEYS
   (PKTABLE_CAT, PKTABLE_SCHEM, PKTABLE_NAME, PKCOLUMN_NAME,
    FKTABLE_CAT, FKTABLE_SCHEM, FKTABLE_NAME, FKCOLUMN_NAME,
    KEY_SEQ, UPDATE_RULE, DELETE_RULE, FK_NAME, PK_NAME, DEFERABILITY)
as
select NULL, po.name, pb.name, pcl.name,
       NULL, fo.name, fb.name, fcl.name,
       fcc.pos#, 
       1,
       decode(pcd.refact, 1, 0, 1),
       fco.name,
       pco.name,
       0
from sys.obj$  fb,
     sys.cdef$ fcd,
     sys.ccol$ fcc,
     sys.col$  fcl,
     sys.con$  fco,
     sys.user$ fo,
     sys.obj$  pb,
     sys.cdef$ pcd,
     sys.ccol$ pcc,
     sys.col$  pcl,
     sys.con$  pco,
     sys.user$ po
where fo.user# = fb.owner#
  and fb.type# = 2
  and fcd.enabled = 1
  and fcd.obj# = fb.obj#
  and fcc.con# = fcd.con#
  and fco.con# = fcd.con#
  and fcd.type# = 4
  and fcl.obj# = fcc.obj#
  and fcl.col# = fcc.col#
  and fcd.rcon# = pcd.con#
  and po.user# = pb.owner#
  and pb.type# = 2
  and pcd.obj# = pb.obj#
  and pcc.con# = pcd.con#
  and pco.con# = pcd.con#
  and pcl.obj# = pcc.obj#
  and pcl.col# = pcc.col#
  and pcc.pos# = fcc.pos#
  and (fb.owner# = uid
       or fb.obj# in (select obj#
                      from sys.objauth$
                      where grantee# in (select kzsrorol from x$kzsro))
      )
  and (pb.owner# = uid
       or pb.obj# in (select obj#
                      from sys.objauth$
                      where grantee# in (select kzsrorol from x$kzsro))
      )
/
comment on table OPENLINK$SQL_FOREIGN_KEYS is
'SQLForeignKeys for the OpenLink ODBC database agent'
/
grant select on OPENLINK$SQL_FOREIGN_KEYS to public with grant option
/
drop public synonym OPENLINK$SQL_FOREIGN_KEYS
/
create public synonym OPENLINK$SQL_FOREIGN_KEYS for OPENLINK$SQL_FOREIGN_KEYS
/
commit
/
create or replace view OPENLINK$SQL_PRIMARY_KEYS
  (TABLE_CAT, TABLE_SCHEM, TABLE_NAME, COLUMN_NAME, POSITION, PK_NAME)
as
select NULL, po.name, pb.name, pcl.name, pcc.pos#, pcn.name
from sys.obj$ pb,
     sys.cdef$ pcd,
     sys.ccol$ pcc,
     sys.col$ pcl,
     sys.user$ po,
     sys.con$ pcn
where po.user# = pb.owner#
  and pb.type# = 2
  and pcd.type# = 2
  and pcd.obj# = pb.obj#
  and pcc.con# = pcd.con#
  and pcl.obj# = pcc.obj#
  and pcl.col# = pcc.col#
  and (pb.owner# = uid
       or pb.obj# in (select obj#
                      from sys.objauth$
                      where grantee# in (select kzsrorol from x$kzsro))
      )
  and pcn.con# = pcd.con#
/
comment on table OPENLINK$SQL_PRIMARY_KEYS is
'SQLPrimaryKeys for the OpenLink ODBC database agent'
/
grant select on OPENLINK$SQL_PRIMARY_KEYS to public with grant option
/
drop public synonym OPENLINK$SQL_PRIMARY_KEYS
/
create public synonym OPENLINK$SQL_PRIMARY_KEYS for OPENLINK$SQL_PRIMARY_KEYS
/
commit
/
create or replace view OPENLINK$SQL_SPEC_COLUMNS
  (SCOPE, COLUMN_NAME, DATA_TYPE, TYPE_NAME, COLUMN_SIZE, BUFFER_LENGTH, 
   DECIMAL_DIGITS, PSEUDO_COLUMN, INT_ID, INT_COLS, INT_POS, TABLE_OWNER,
   TABLE_NAME, INDEX_NAME)
as
select 2, c.name, c.type#,
       decode(c.type#, 1, decode(c.charsetform, 2, 'NVARCHAR2', 'VARCHAR2'),
                       2, decode(c.scale, null,
                                 decode(c.precision#, null, 'NUMBER', 'FLOAT'),
                                 'NUMBER'),
                       8, 'LONG',
                       9, decode(c.charsetform, 2, 'NCHAR VARYING', 'VARCHAR'),
                       12, 'DATE',
                       23, 'RAW', 24, 'LONG RAW',
                       58, nvl2(ac.synobj#, (select o.name from obj$ o
                                where o.obj#=ac.synobj#), t.name),
                       69, 'ROWID',
                       96, decode(c.charsetform, 2, 'NCHAR', 'CHAR'),
                       100, 'BINARY_FLOAT',
                       101, 'BINARY_DOUBLE',
                       105, 'MLSLABEL',
                       106, 'MLSLABEL',
                       111, nvl2(ac.synobj#, (select o.name from obj$ o
                                 where o.obj#=ac.synobj#), t.name),
                       112, decode(c.charsetform, 2, 'NCLOB', 'CLOB'),
                       113, 'BLOB', 114, 'BFILE', 115, 'CFILE',
                       121, nvl2(ac.synobj#, (select o.name from obj$ o
                                 where o.obj#=ac.synobj#), t.name),
                       122, nvl2(ac.synobj#, (select o.name from obj$ o
                                 where o.obj#=ac.synobj#), t.name),
                       123, nvl2(ac.synobj#, (select o.name from obj$ o
                                 where o.obj#=ac.synobj#), t.name),
                       178, 'TIME(' ||c.scale|| ')',
                       179, 'TIME(' ||c.scale|| ')' || ' WITH TIME ZONE',
                       180, 'TIMESTAMP(' ||c.scale|| ')',
                       181, 'TIMESTAMP(' ||c.scale|| ')' || ' WITH TIME ZONE',
                       231, 'TIMESTAMP(' ||c.scale|| ')' || ' WITH LOCAL TIME ZONE',
                       182, 'INTERVAL YEAR(' ||c.precision#||') TO MONTH',
                       183, 'INTERVAL DAY(' ||c.precision#||') TO SECOND(' ||
                             c.scale || ')',
                       208, 'UROWID',
                       'UNDEFINED'),
    c.precision#, c.length, c.scale, 1, 1, i.cols, ic.pos#, towner.name, t.name,
    i_o.name
from
    sys.obj$ t,
    sys.col$ c,
    sys.coltype$ ac,
    sys.ind$ i,
    sys.icol$ ic,
    sys.obj$ i_o,
    sys.user$ towner
where t.type# = 2 
  and towner.user# = t.owner#
  and i.bo# = t.obj# 
  and i.property = 1 
  and i_o.obj# = i.obj# 
  and ic.obj# = i_o.obj# 
  and ic.col# = c.col# 
  and c.obj# = t.obj# 
  and ac.toid = t.oid$(+)
  and (t.owner# = uid
       or t.obj# in (select obj#
                      from sys.objauth$
                      where grantee# in (select kzsrorol from x$kzsro))
      )
union
select 1, 'ROWID', 1, 'CHAR', 18, 18, 0, 1, 2, 1, 1, towner.name, 
  t.name, ''
from
    sys.obj$ t, 
    sys.user$ towner
where t.type# = 2 
  and towner.user# = t.owner#
  and (t.owner# = uid
       or t.obj# in (select obj#
                      from sys.objauth$
                      where grantee# in (select kzsrorol from x$kzsro))
      )
/
comment on table OPENLINK$SQL_SPEC_COLUMNS is
'SQLSpecialColumns for the OpenLink ODBC database agent'
/
grant select on OPENLINK$SQL_SPEC_COLUMNS to public with grant option
/
drop public synonym OPENLINK$SQL_SPEC_COLUMNS
/
create public synonym OPENLINK$SQL_SPEC_COLUMNS for OPENLINK$SQL_SPEC_COLUMNS
/
commit
/

Rem o.type# - 9 is packaged objects, 7 is a stored function, 8 is a stored procedure

create or replace view OPENLINK$SQL_PROC_COLUMNS
  (PROCEDURE_CAT,PROCEDURE_SCHEM,PROCEDURE_NAME,COLUMN_NAME,
   COLUMN_TYPE,DATA_TYPE,TYPE_NAME,COLUMN_SIZE,BUFFER_LENGTH,DECIMAL_DIGITS,
   NUM_PREC_RADIX,NULLABLE,REMARKS,COLUMN_DEF,SQL_DATA_TYPE,SQL_DATE_TIME_SUB,
   CHAR_OCTET_LENGTH,ORDINAL_POSITION,IS_NULLABLE)
as
select NULL, 
    u.name, 
    t.name,
    decode(a.position#,
        0,'RETURN_VALUE',
        a.argument),
    decode(a.in_out,
        NULL,1, 
	2,2,
	1,decode(a.position#,
	      0,5,
	      4),
	1),        
    a.type#,
    decode(c.type#, 1, decode(c.charsetform, 2, 'NVARCHAR2', 'VARCHAR2'),
                    2, decode(c.scale, null,
                                 decode(c.precision#, null, 'NUMBER', 'FLOAT'),
                                 'NUMBER'),
                    8, 'LONG',
                    9, decode(c.charsetform, 2, 'NCHAR VARYING', 'VARCHAR'),
                    12, 'DATE',
                    23, 'RAW', 24, 'LONG RAW',
                    58, nvl2(ac.synobj#, (select o.name from obj$ o
                                where o.obj#=ac.synobj#), t.name),
                    69, 'ROWID',
                    96, decode(c.charsetform, 2, 'NCHAR', 'CHAR'),
                    100, 'BINARY_FLOAT',
                    101, 'BINARY_DOUBLE',
                    105, 'MLSLABEL',
                    106, 'MLSLABEL',
                    111, nvl2(ac.synobj#, (select o.name from obj$ o
                              where o.obj#=ac.synobj#), t.name),
                    112, decode(c.charsetform, 2, 'NCLOB', 'CLOB'),
                    113, 'BLOB', 
		    114, 'BFILE', 
		    115, 'CFILE',
                    121, nvl2(ac.synobj#, (select o.name from obj$ o
                              where o.obj#=ac.synobj#), t.name),
		    122, nvl2(ac.synobj#, (select o.name from obj$ o
                              where o.obj#=ac.synobj#), t.name),
                    123, nvl2(ac.synobj#, (select o.name from obj$ o
                              where o.obj#=ac.synobj#), t.name),
                    178, 'TIME(' ||c.scale|| ')',
                    179, 'TIME(' ||c.scale|| ')' || ' WITH TIME ZONE',
                    180, 'TIMESTAMP(' ||c.scale|| ')',
                    181, 'TIMESTAMP(' ||c.scale|| ')' || ' WITH TIME ZONE',
                    231, 'TIMESTAMP(' ||c.scale|| ')' || ' WITH LOCAL TIME ZONE',
                    182, 'INTERVAL YEAR(' ||c.precision#||') TO MONTH',
                    183, 'INTERVAL DAY(' ||c.precision#||') TO SECOND(' ||
                             c.scale || ')',
                    208, 'UROWID',
                    'UNDEFINED'),
    decode(a.precision#,
        null,decode(a.type#,
                 1,255,
		 255),
  	a.precision#),
    a.length, 
    a.scale, 
    a.radix, 
    2,
    NULL,
    to_char(a.default#),
    0,
    0,
    decode (a.type#,
         96, a.length, 1, a.length, NULL),
    a.position#,
    ''
from
    sys.obj$ t,
    sys.col$ c,
    sys.coltype$ ac,
    sys.argument$ a,
    sys.user$ u
where
    u.user# = t.owner# and
    a.obj#=t.obj# and
    t.type# in (7,8) and 
    c.obj# = t.obj#  and
    ac.toid = t.oid$(+) and
    t.status=1
UNION
select NULL, 
    u.name, 
    t.name || '.' || a.procedure$,
    decode(a.position#,
        0,'RETURN_VALUE',
        a.argument),
    decode(a.in_out,
        NULL,1, 
	2,2,
	1,decode(a.position#,
	      0,5,
	      4),
	1),        
    a.type#,
    decode(c.type#, 1, decode(c.charsetform, 2, 'NVARCHAR2', 'VARCHAR2'),
                    2, decode(c.scale, null,
                                 decode(c.precision#, null, 'NUMBER', 'FLOAT'),
                                 'NUMBER'),
                    8, 'LONG',
                    9, decode(c.charsetform, 2, 'NCHAR VARYING', 'VARCHAR'),
                    12, 'DATE',
                    23, 'RAW', 24, 'LONG RAW',
                    58, nvl2(ac.synobj#, (select o.name from obj$ o
                                where o.obj#=ac.synobj#), t.name),
                    69, 'ROWID',
                    96, decode(c.charsetform, 2, 'NCHAR', 'CHAR'),
                    100, 'BINARY_FLOAT',
                    101, 'BINARY_DOUBLE',
                    105, 'MLSLABEL',
                    106, 'MLSLABEL',
                    111, nvl2(ac.synobj#, (select o.name from obj$ o
                              where o.obj#=ac.synobj#), t.name),
                    112, decode(c.charsetform, 2, 'NCLOB', 'CLOB'),
                    113, 'BLOB', 
		    114, 'BFILE', 
		    115, 'CFILE',
                    121, nvl2(ac.synobj#, (select o.name from obj$ o
                              where o.obj#=ac.synobj#), t.name),
		    122, nvl2(ac.synobj#, (select o.name from obj$ o
                              where o.obj#=ac.synobj#), t.name),
                    123, nvl2(ac.synobj#, (select o.name from obj$ o
                              where o.obj#=ac.synobj#), t.name),
                    178, 'TIME(' ||c.scale|| ')',
                    179, 'TIME(' ||c.scale|| ')' || ' WITH TIME ZONE',
                    180, 'TIMESTAMP(' ||c.scale|| ')',
                    181, 'TIMESTAMP(' ||c.scale|| ')' || ' WITH TIME ZONE',
                    231, 'TIMESTAMP(' ||c.scale|| ')' || ' WITH LOCAL TIME ZONE',
                    182, 'INTERVAL YEAR(' ||c.precision#||') TO MONTH',
                    183, 'INTERVAL DAY(' ||c.precision#||') TO SECOND(' ||
                             c.scale || ')',
                    208, 'UROWID',
                    'UNDEFINED'),
    decode(a.precision#,
        null,decode(a.type#,
                 1,255,
		 255),
  	a.precision#),
    a.length, 
    a.scale, 
    a.radix, 
    2,
    NULL,
    to_char(a.default#),
    0,
    0,
    decode (a.type#,
         96, a.length, 1, a.length, NULL),
    a.position#,
    ''
from
    sys.obj$ t,
    sys.col$ c,
    sys.coltype$ ac,
    sys.argument$ a,
    sys.user$ u
where
    u.user# = t.owner# and
    a.obj#=t.obj# and
    t.type# = 9 and 
    c.obj# = t.obj#  and
    ac.toid = t.oid$(+) and
    t.status=1
/
comment on table OPENLINK$SQL_PROC_COLUMNS is
'SQLProcedureColumns for the OpenLink ODBC database agent'
/
grant select on OPENLINK$SQL_PROC_COLUMNS to public with grant option
/
drop public synonym OPENLINK$SQL_PROC_COLUMNS
/
create public synonym OPENLINK$SQL_PROC_COLUMNS for OPENLINK$SQL_PROC_COLUMNS
/
commit
/

Rem
Rem  STORED FUNCTIONS FOR SCALAR FUNCTION SUPPORT
Rem

drop package OPENLINK
/

create package OPENLINK as
  function INSERT_STRING(stra in string, strt in number, leng in number, strb in string) return string;
  pragma restrict_references(INSERT_STRING,wnds,wnps);
  function LEFT(stra in string, cnt in number) return string;
  pragma restrict_references(LEFT,wnds,wnps);
  function RIGHT(stra in string, cnt in number) return string;
  pragma restrict_references(RIGHT,wnds,wnps);
  function LOCATE(stra in string, strb in string, strt in number default 1) 
    return number;
  pragma restrict_references(LOCATE,wnds,wnps);
  function REPEAT(strin in string, cnt in number) return string;
  pragma restrict_references(REPEAT,wnds,wnps);
  function RADIANS(degrees in float) return float; 
  pragma restrict_references(RADIANS,wnds,wnps);
  function DEGREES(radians in float) return float;
  pragma restrict_references(DEGREES,wnds,wnps);
  function COT(radians in float) return float;
  pragma restrict_references(COT,wnds,wnps);
  function COUNT_IN_PARMS(obj_id in number) return number;
  pragma restrict_references(COUNT_IN_PARMS,wnds,wnps);
  function COUNT_OUT_PARMS(obj_id in number) return number;
  pragma restrict_references(COUNT_OUT_PARMS,wnds,wnps);
  function PCOUNT_IN_PARMS(obj_id in number, proc in string) return number;
  pragma restrict_references(PCOUNT_IN_PARMS,wnds,wnps);
  function PCOUNT_OUT_PARMS(obj_id in number, proc in string) return number;
  pragma restrict_references(PCOUNT_OUT_PARMS,wnds,wnps);
  function PACK_OBJ_TYPE(obj_id in number, proc in string) return number;
  pragma restrict_references(PACK_OBJ_TYPE,wnds,wnps);
  function TIMESTAMPADD(interval in string, integer_exp in integer,
    timestamp_exp in date) return date;
  function TIMESTAMPADD(interval in string, integer_exp in integer,
    timestamp_exp in string) return date;
  pragma restrict_references (TIMESTAMPADD, wnds, wnps);
  function TIMESTAMPDIFF(interval in string, timestamp_exp1 in date,
    timestamp_exp2 in date) return integer;
  function TIMESTAMPDIFF(interval in string, timestamp_exp1 in date,
    timestamp_exp2 in string) return integer;
  function TIMESTAMPDIFF (interval in string, timestamp_exp1 in string,
    timestamp_exp2 in date) return integer;
  function TIMESTAMPDIFF (interval in string, timestamp_exp1 in string,
    timestamp_exp2 in string) return integer;
  pragma restrict_references (TIMESTAMPDIFF, wnds, wnps);
end;
/

create package body OPENLINK as

  ------------------------------------------------------------------------
  -- local functions
  --

  -- TIMESTAMPADD_DATE:
  -- Returns the timestamp calculated by adding integer_exp intervals of the
  -- type indicated by interval to timestamp_exp.

  function TIMESTAMPADD_DATE (
    interval in string,
    integer_exp in integer,
    timestamp_exp in date
    )
    return date
  as
    nuDate date;
  begin

    -- Oracle interprets number constants in arithmetic date expressions as
    -- numbers of days.

    if interval = 'SQL_TSI_FRAC_SECOND' then
      -- integer_exp multiplier = 1/billionth's of a second in a day
      nuDate := timestamp_exp + (integer_exp / (86400 * 1e9));
    elsif interval = 'SQL_TSI_SECOND' then
      -- integer_exp multiplier := 1/seconds in a day
      nuDate := timestamp_exp + (integer_exp / 86400);
    elsif interval = 'SQL_TSI_MINUTE' then
      nuDate := timestamp_exp + (integer_exp / 1440);
    elsif interval = 'SQL_TSI_HOUR' then
      nuDate := timestamp_exp + (integer_exp / 24);
    elsif interval = 'SQL_TSI_DAY' then
      nuDate := timestamp_exp + integer_exp;
    elsif interval = 'SQL_TSI_WEEK' then
      nuDate := timestamp_exp + (7 * integer_exp);
    elsif interval = 'SQL_TSI_MONTH' then
      nuDate := add_months (timestamp_exp, integer_exp);
    elsif interval = 'SQL_TSI_QUARTER' then
      nuDate := add_months (timestamp_exp, 3 * integer_exp);
    elsif interval = 'SQL_TSI_YEAR' then
      nuDate := add_months (timestamp_exp, 12 * integer_exp);
    else
      raise_application_error (-20000, 
	  'TIMESTAMPADD: Invalid SQL_TSI_xxx argument');
    end if;

    return nuDate;
  end;

  -- TIMESTAMPDIFF_DATE:
  -- Returns the integer number of intervals of the type indicated by interval
  -- by which timestamp_exp2 is greater than timestamp_exp1

  function TIMESTAMPDIFF_DATE (
    interval in string,
    timestamp_exp1 in date,
    timestamp_exp2 in date
    )
    return integer
  as
    daysDiff number;
    monthsDiff number;
    cIntervals integer;
  begin

    daysDiff := timestamp_exp2 - timestamp_exp1;
    monthsDiff := months_between (timestamp_exp2, timestamp_exp1);

    -- Oracle expresses the difference between two DATE values in
    -- numbers of days. Convert the number of days into an integral number of
    -- intervals of the specified type.

    if interval = 'SQL_TSI_FRAC_SECOND' then
      -- multiplier = billionth's of a second in a day
      cIntervals := sign (daysDiff) * floor (abs (daysDiff) * 86400 * 1e9);
    elsif interval = 'SQL_TSI_SECOND' then
      -- multiplier = seconds in a day
      cIntervals := sign (daysDiff) * floor (abs (daysDiff) * 86400);
    elsif interval = 'SQL_TSI_MINUTE' then
      cIntervals := sign (daysDiff) * floor (abs (daysDiff) * 1440);
    elsif interval = 'SQL_TSI_HOUR' then
      cIntervals := sign (daysDiff) * floor (abs (daysDiff) * 24);
    elsif interval = 'SQL_TSI_DAY' then
      cIntervals := sign (daysDiff) * floor (abs (daysDiff));
    elsif interval = 'SQL_TSI_WEEK' then
      cIntervals := sign (daysDiff) * floor (abs (daysDiff) / 7);
    elsif interval = 'SQL_TSI_MONTH' then
      cIntervals := sign (monthsDiff) * floor (abs (monthsDiff));
    elsif interval = 'SQL_TSI_QUARTER' then
      cIntervals := sign (monthsDiff) * floor (abs (monthsDiff) / 3);
    elsif interval = 'SQL_TSI_YEAR' then
      cIntervals := sign (monthsDiff) * floor (abs (monthsDiff) / 12);
    else
      raise_application_error (-20000, 
	  'TIMESTAMPDIFF: Invalid SQL_TSI_xxx argument');
    end if;

    return cIntervals;
  end;
  
  -- strToDate:
  -- Converts an ODBC date, time or timestamp literal into an Oracle DATE

  function strToDate (
    timestamp_exp in string
    )
    return date
  as
    timestamp date;
    dayOfMonth integer;
    charIndx integer;
  begin

    -- Determine whether we have a date, time or timestamp literal

    if instr (timestamp_exp, '-', 1, 2) = 0 then
      -- Assume there's no date component present.
      -- i.e. we have an ODBC time literal
      timestamp := to_date (timestamp_exp, 'HH24:MI:SS');

      -- Oracle defaults date component of resulting date to first day of
      -- current month. Set date component to current date.
      dayOfMonth := to_number (to_char (sysdate,'DD'));
      timestamp := timestamp + (dayOfMonth - 1);

    elsif instr (timestamp_exp, ':', 1, 2) = 0 then
      -- Assume there's no time component present.
      -- i.e. we have an ODBC date literal
      -- Oracle defaults time component of resulting date to 00:00:00
      timestamp := to_date (timestamp_exp, 'YYYY-MM-DD');

    else
      -- We have an ODBC timestamp literal.
      -- Oracle DATEs don't support fractional seconds.
      -- Strip off any fractional seconds in the timestamp literal, otherwise
      -- the conversion will fail.
      charIndx := instr (timestamp_exp, '.');
      if charIndx > 0 then
        timestamp := to_date (substr (timestamp_exp, 1, charIndx - 1), 
	  		      'YYYY-MM-DD HH24:MI:SS');
      else
        timestamp := to_date (timestamp_exp, 'YYYY-MM-DD HH24:MI:SS');
      end if;

    end if;

    return timestamp;
  end;

  ------------------------------------------------------------------------
  -- public functions
  --

  function INSERT_STRING(stra in string, strt in number, leng in number, strb in string)
  return string as
  begin
    return SUBSTR(stra,1,strt-1) || strb || SUBSTR(stra,strt+leng);
  end;

  function LEFT(stra in string, cnt in number)
  return string as
  begin
    return SUBSTR(stra,1,cnt);
  end;

  function RIGHT(stra in string, cnt in number)
  return string as
  begin
    return SUBSTR(stra,(LENGTH(stra)-cnt)+1);
  end;

  function LOCATE(stra in string, strb in string, strt in number default 1)
  return number as
  begin
    return INSTR(strb,stra,strt);
  end;

  function REPEAT(strin in string, cnt in number)
  return string as
  begin
    declare 
      tally number;
      outstring string(2000);
    begin
      tally := cnt;
      WHILE tally!=0 LOOP
        outstring := outstring || strin;
        tally := tally-1;
      END LOOP;
      return outstring;
    end;
  end;
 
  function RADIANS(degrees in float) 
  return float as
  begin
    return (degrees/360)*2*3.1415926545;
  end;

  function DEGREES(radians in float)
  return float as
  begin
    return (radians/(2*3.141592654))*360;
  end;
  
  function COT(radians in float)
  return float as
  begin
	return (1/tan(radians));
  end;
 
  function COUNT_IN_PARMS(obj_id in number)
  return number as
    nparms NUMBER;
  begin
    select count(*) into nparms 
      from   argument$ a
      where  a.obj#=obj_id
        and  in_out is null;
    if nparms is null then
      nparms:=0;
    end if;
    return nparms;
  end;

  function COUNT_OUT_PARMS(obj_id in number)
  return number as
    nparms NUMBER;
  begin
    select count(*) into nparms 
      from   argument$ a
      where  a.obj#=obj_id
        and  in_out=1;
    if nparms is null then
      nparms:=0;
    end if;
    return nparms;
  end;

  function PCOUNT_IN_PARMS(obj_id in number, proc in string)
  return number as
    nparms NUMBER;
  begin
    select count(*) into nparms 
      from   argument$ a, obj$ o
      where  a.obj#=obj_id
        and  a.obj#=o.obj#
	and  o.type#=9
        and  a.procedure$=proc
        and  in_out is null;
    if nparms is null then
      nparms:=0;
    end if;
    return nparms;
  end;

  function PCOUNT_OUT_PARMS(obj_id in number, proc in string)
  return number as
    nparms NUMBER;
  begin
    select count(*) into nparms 
      from   argument$ a, obj$ o
      where  a.obj#=obj_id
        and  a.obj#=o.obj#
	and  o.type#=9
        and  a.procedure$=proc
        and  in_out=1;
    if nparms is null then
      nparms:=0;
    end if;
    return nparms;
  end;

  function PACK_OBJ_TYPE(obj_id in number, proc in string)
  return number as
    objtype# NUMBER;
  begin
    begin
      select a.position# into objtype# 
        from   argument$ a, obj$ o
          where  a.obj#=obj_id
            and  a.procedure$=proc
            and  o.obj#=a.obj#
	    and  o.type#=9
            and  a.position#=0
            and  a.in_out=1;
    exception
      when too_many_rows then
        objtype#:=1;
    end;
    if objtype#=0 then
      objtype#:=2;
    else
      objtype#:=1;
    end if;
    return objtype#;
  end;

  function TIMESTAMPADD (
    interval in string,
    integer_exp in integer,
    timestamp_exp in date
    )
    return date
  as
  begin
    return TIMESTAMPADD_DATE (interval, integer_exp, timestamp_exp);
  end;

  function TIMESTAMPADD (
    interval in string,
    integer_exp in integer,
    timestamp_exp in string
    )
    return date
  as
  begin
    return TIMESTAMPADD_DATE (interval, integer_exp, 
			      strToDate (timestamp_exp));
  end;

  function TIMESTAMPDIFF (
    interval in string,
    timestamp_exp1 in date,
    timestamp_exp2 in date
    )
    return integer
  as
  begin
    return TIMESTAMPDIFF_DATE (interval, timestamp_exp1, timestamp_exp2);
  end;


  function TIMESTAMPDIFF (
    interval in string,
    timestamp_exp1 in date,
    timestamp_exp2 in string
    )
    return integer
  as
  begin
    return TIMESTAMPDIFF_DATE (interval, timestamp_exp1, 
			       strToDate (timestamp_exp2));
  end;

  function TIMESTAMPDIFF (
    interval in string,
    timestamp_exp1 in string,
    timestamp_exp2 in date
    )
    return integer
  as
  begin
    return TIMESTAMPDIFF_DATE (interval, strToDate (timestamp_exp1), 
			       timestamp_exp2);
  end;

  function TIMESTAMPDIFF (
    interval in string,
    timestamp_exp1 in string,
    timestamp_exp2 in string
    )
    return integer
  as
  begin
    return TIMESTAMPDIFF_DATE (interval, strToDate (timestamp_exp1), 
			       strToDate (timestamp_exp2));
  end;

end openlink;
/

show errors

Rem
Rem  NOTE: We use the oplora7 name for now, to be compatible with the 
Rem        oracle 7 agent. In future we may change the name.
Rem
drop public synonym OPLORA7
/
create public synonym OPLORA7 for sys.openlink
/
grant execute on sys.openlink to public with grant option
/
alter package openlink COMPILE
/

Rem 
Rem  Note: Function type# for Packaged Objects not supported as yet.
Rem  Awaiting feedback from Oracle Corp on outstanding issue.
Rem
create or replace view OPENLINK$SQL_PROCEDURES
  (PROCEDURE_CAT,PROCEDURE_SCHEM,PROCEDURE_NAME,NUM_INPUT_PARAMS,
   NUM_OUTPUT_PARAMS,NUM_RESULT_SETS,REMARKS,PROCEDURE_TYPE)
as
select NULL,
    u.name,
    o.name,
    openlink.count_in_parms(o.obj#),
    openlink.count_out_parms(o.obj#),
    -1,
    NULL,
    decode(o.type#,7,1,8,2,0)
from
    sys.obj$ o,
    sys.user$ u
where
    u.user#=o.owner# and
    o.type# in (7,8) and
    o.status=1
union
select NULL,
    u1.name,
    o1.name || '.' || a1.procedure$,
    openlink.pcount_in_parms(o1.obj#,a1.procedure$),
    openlink.pcount_out_parms(o1.obj#,a1.procedure$),
    -1,
    NULL,
    0
from
    user$ u1,
    obj$ o1,
    argument$ a1
where
    u1.user#=o1.owner# and
    a1.obj#=o1.obj# and
    o1.type#=9 and
    o1.status=1
/
comment on table OPENLINK$SQL_PROCEDURES is
'SQLProcedures for the OpenLink ODBC database agent'
/
grant select on OPENLINK$SQL_PROCEDURES to public with grant option
/
drop public synonym OPENLINK$SQL_PROCEDURES
/
create public synonym OPENLINK$SQL_PROCEDURES for OPENLINK$SQL_PROCEDURES
/

commit
/


Rem
Rem  Note: Function type# for Packaged Objects not supported as yet.
Rem  Awaiting feedback from Oracle Corp on outstanding issue.
Rem
create or replace view OPENLINK$SQL_PROCS_NOCOUNT
  (PROCEDURE_CAT,PROCEDURE_SCHEM,PROCEDURE_NAME,NUM_INPUT_PARAMS,
   NUM_OUTPUT_PARAMS,NUM_RESULT_SETS,REMARKS,PROCEDURE_TYPE)
as
select NULL,
    u.name,
    o.name,
    -1,
    -1,
    -1,
    NULL,
    decode(o.type#,7,1,8,2,0)
from
    sys.obj$ o,
    sys.user$ u
where
    u.user#=o.owner# and
    o.type# in (7,8) and
    o.status=1
union
select NULL,
    u1.name,
    o1.name || '.' || a1.procedure$,
    -1,
    -1,
    -1,
    NULL,
    0
from
    user$ u1,
    obj$ o1,
    argument$ a1
where
    u1.user#=o1.owner# and
    a1.obj#=o1.obj# and
    o1.type#=9 and
    o1.status=1
/
comment on table OPENLINK$SQL_PROCS_NOCOUNT is
'SQLProcedures for the OpenLink ODBC database agent with no support for counting params'
/
grant select on OPENLINK$SQL_PROCS_NOCOUNT to public with grant option
/
drop public synonym OPENLINK$SQL_PROCS_NOCOUNT
/
create public synonym OPENLINK$SQL_PROCS_NOCOUNT for OPENLINK$SQL_PROCS_NOCOUNT
/

commit
/

Rem
Rem  Create table OPLRVC, the row versioning cache required for 
Rem  dynamic cursor support.
Rem

drop table OPLRVC
/
create table OPLRVC
(HCONN CHARACTER(8),
 HCURS CHARACTER(8),
 KEYVER CHARACTER(32),
 ROWVER CHARACTER(32))
/
CREATE INDEX OPLRVCIX ON OPLRVC (HCONN, HCURS, KEYVER, ROWVER)
/
GRANT ALL ON OPLRVC TO PUBLIC
/
comment on table OPLRVC is
'Row versioning cache for the OpenLink ODBC database agent'
/
drop public synonym OPLRVC
/
create public synonym OPLRVC for OPLRVC
/

commit
/
